

#include "clock.h"


void system_clock_config(void)
{
  unsigned int tmp=0;

#if CPU_HZ == 80000000
  tmp = FLASH->ACR;  /* flash access control register */
  clear_bit(tmp, 0);
  clear_bit(tmp, 1); /* set flash access latency to four wait states */
  set_bit(tmp, 2);
  set_bit(tmp, 8);   /* enable prefetch */
  FLASH->ACR = tmp;  /* flash access control register */
#elif CPU_HZ == 32000000
  tmp = FLASH->ACR;  /* flash access control register */
  set_bit(tmp, 0);
  clear_bit(tmp, 1); /* set flash access latency to one wait state */
  clear_bit(tmp, 2);
  clear_bit(tmp, 8); /* disable prefetch */
  FLASH->ACR = tmp;  /* flash access control register */
#elif (CPU_HZ == 16000000) || (CPU_HZ == 16384000) || (CPU_HZ == 8192000) || (CPU_HZ == 8000000)
  tmp = FLASH->ACR;  /* flash access control register */
  clear_bit(tmp, 0);
  clear_bit(tmp, 1); /* set flash access latency to zero wait states */
  clear_bit(tmp, 2);
  clear_bit(tmp, 8); /* disable prefetch */
  FLASH->ACR = tmp;  /* flash access control register */
#else
#error "Invalid value for CPU_HZ!"
#endif
  _NOP();
  _NOP();
  _NOP();

  set_bit(RCC->APB2ENR, 0);  /* SYSCFG + COMP + VREFBUF clock enable */
  _NOP();
  _NOP();
  _NOP();

  set_bit(RCC->APB1ENR1, 28);  /* enable power, APB1 peripheral clock enable register 1 */
  _NOP();
  _NOP();
  _NOP();

  tmp = PWR->CR1;  /* Power control register 1 */
  set_bit(tmp, 8);  /* disable write protection for the RTC and backup registers */
  set_bit(tmp, 9);  /* Voltage scaling range selection to range 1 */
  clear_bit(tmp, 10);
  PWR->CR1 = tmp;  /* Power control register 1 */
  _NOP();
  _NOP();
  _NOP();

  set_bit(RCC->CR, 8);  /* clock control register: enable HSI16 oscillator */
  _NOP();
  _NOP();
  _NOP();
  while(!test_bit(RCC->CR, 10)) ;  /* clock control register: wait till HSI16 oscillator is ready */

  set_bit(RCC->CRRCR, 0);  /* clock recovery RC register: enable RC48 oscillator */
  _NOP();
  _NOP();
  _NOP();
  while(!test_bit(RCC->CRRCR, 1)) ;  /* wait till 48MHz clock is ready */

  set_bit(CRS->CR, 6);  /* CRS control register: enable automatic trimming of the RC48 oscillator */
  _NOP();
  _NOP();
  _NOP();

#if CPU_HZ == 80000000
  tmp = RCC->PLLCFGR;  /* PLL configuration register */
  clear_bit(tmp, 0);   /* use HSI16 clock for PLL source */
//  set_bit(tmp, 0);   /* use HSE clock for PLL source */
  set_bit(tmp, 1);
//  clear_bit(tmp, 4);   /* PLLM divider set to 1 */
  set_bit(tmp, 4);     /* PLLM divider set to 2 */
  clear_bit(tmp, 5);
  clear_bit(tmp, 6);

  clear_bit(tmp, 8);   /* PLLN multiplier set to 20 */
  clear_bit(tmp, 9);
  set_bit(tmp, 10);
  clear_bit(tmp, 11);
  set_bit(tmp, 12);
  clear_bit(tmp, 13);
  clear_bit(tmp, 14);
  clear_bit(tmp, 16);  /* disable PLLP output */
  clear_bit(tmp, 17);  /* set PLLP divider to 7 */
  clear_bit(tmp, 20);  /* disable 48M1CLK output */
  clear_bit(tmp, 21);  /* set PLLQ divider to 2 */
  clear_bit(tmp, 22);
  set_bit(tmp, 24);    /* enable PLLR clock output */
  clear_bit(tmp, 25);  /* set PLLR divider to 2 */
  clear_bit(tmp, 26);
  set_bit(tmp, 27);    /* set PLLP divider to 7 */
  set_bit(tmp, 28);
  set_bit(tmp, 29);
  clear_bit(tmp, 30);
  clear_bit(tmp, 31);
  RCC->PLLCFGR = tmp;  /* PLL configuration register */
#elif CPU_HZ == 32000000
  tmp = RCC->PLLCFGR;  /* PLL configuration register */
  clear_bit(tmp, 0);   /* use HSI16 clock for PLL source */
//  set_bit(tmp, 0);     /* use HSE clock for PLL source */
  set_bit(tmp, 1);
  set_bit(tmp, 4);     /* PLLM divider set to 2 */
//  clear_bit(tmp, 4);   /* PLLM divider set to 1 */
  clear_bit(tmp, 5);
  clear_bit(tmp, 6);

  clear_bit(tmp, 8);   /* PLLN multiplier set to 8 */
  clear_bit(tmp, 9);
  clear_bit(tmp, 10);
  set_bit(tmp, 11);
  clear_bit(tmp, 12);
  clear_bit(tmp, 13);
  clear_bit(tmp, 14);
  clear_bit(tmp, 16);  /* disable PLLP output */
  clear_bit(tmp, 17);  /* set PLLP divider to 7 */
  clear_bit(tmp, 20);  /* disable 48M1CLK output */
  clear_bit(tmp, 21);  /* set PLLQ divider to 2 */
  clear_bit(tmp, 22);
  set_bit(tmp, 24);    /* enable PLLR clock output */
  clear_bit(tmp, 25);  /* set PLLR divider to 2 */
  clear_bit(tmp, 26);
  set_bit(tmp, 27);    /* set PLLP divider to 7 */
  set_bit(tmp, 28);
  set_bit(tmp, 29);
  clear_bit(tmp, 30);
  clear_bit(tmp, 31);
  RCC->PLLCFGR = tmp;  /* PLL configuration register */
#elif (CPU_HZ == 16000000) || (CPU_HZ == 16384000)
  tmp = RCC->PLLCFGR;  /* PLL configuration register */
  clear_bit(tmp, 0);   /* use HSI16 clock for PLL source */
//  set_bit(tmp, 0);     /* use HSE clock for PLL source */
  set_bit(tmp, 1);
  set_bit(tmp, 4);     /* PLLM divider set to 2 */
//  clear_bit(tmp, 4);   /* PLLM divider set to 1 */
  clear_bit(tmp, 5);
  clear_bit(tmp, 6);

  clear_bit(tmp, 8);   /* PLLN multiplier set to 8 */
  clear_bit(tmp, 9);
  clear_bit(tmp, 10);
  set_bit(tmp, 11);
  clear_bit(tmp, 12);
  clear_bit(tmp, 13);
  clear_bit(tmp, 14);
  clear_bit(tmp, 16);  /* disable PLLP output */
  clear_bit(tmp, 17);  /* set PLLP divider to 7 */
  clear_bit(tmp, 20);  /* disable 48M1CLK output */
  clear_bit(tmp, 21);  /* set PLLQ divider to 2 */
  clear_bit(tmp, 22);
  set_bit(tmp, 24);    /* enable PLLR clock output */
  set_bit(tmp, 25);    /* set PLLR divider to 4 */
  clear_bit(tmp, 26);
  set_bit(tmp, 27);    /* set PLLP divider to 7 */
  set_bit(tmp, 28);
  set_bit(tmp, 29);
  clear_bit(tmp, 30);
  clear_bit(tmp, 31);
  RCC->PLLCFGR = tmp;  /* PLL configuration register */
#else
#error "Invalid value for CPU_HZ!"
#endif
  _NOP();
  _NOP();
  _NOP();

#if (CPU_HZ == 80000000) || (CPU_HZ == 32000000) || (CPU_HZ == 16000000) || (CPU_HZ == 16384000)
  set_bit(RCC->CR, 24);  /* clock control register: enable main PLL */
  _NOP();
  _NOP();
  _NOP();
  while(!test_bit(RCC->CR, 25)) ;  /* clock control register: wait till PLL is locked */
#elif (CPU_HZ == 8000000) || (CPU_HZ == 8192000)
    /* nothing here */
#else
#error "Invalid value for CPU_HZ!"
#endif

#if CPU_HZ == 80000000
  clear_bit(RCC->CFGR, 8);   /* clock configuration register: set to APB1 divider to 2 */
  clear_bit(RCC->CFGR, 9);
  set_bit(RCC->CFGR, 10);

  clear_bit(RCC->CFGR, 11);  /* clock configuration register: set to APB2 divider to 2 */
  clear_bit(RCC->CFGR, 12);
  set_bit(RCC->CFGR, 13);
#elif (CPU_HZ == 32000000) || (CPU_HZ == 16000000) || (CPU_HZ == 8000000) || (CPU_HZ == 16384000) || (CPU_HZ == 8192000)
  clear_bit(RCC->CFGR, 8);   /* clock configuration register: set to APB1 divider to 1 */
  clear_bit(RCC->CFGR, 9);
  clear_bit(RCC->CFGR, 10);

  clear_bit(RCC->CFGR, 11);  /* clock configuration register: set to APB2 divider to 1 */
  clear_bit(RCC->CFGR, 12);
  clear_bit(RCC->CFGR, 13);
#else
#error "Invalid value for CPU_HZ!"
#endif

  clear_bit(RCC->CFGR, 7);  /* clock configuration register: set to AHB divider to 1 */

//   tmp = RCC->CFGR;  /* clock configuration register */
// #if CPU_HZ == 80000000
//   clear_bit(tmp, 28);
//   clear_bit(tmp, 29);  /* MCO is divided by 16 */
//   set_bit(tmp, 30);
// #elif CPU_HZ == 16000000
//   clear_bit(tmp, 28);
//   set_bit(tmp, 29);    /* MCO is divided by 4 */
//   clear_bit(tmp, 30);
// #else
// #error "Invalid value for CPU_HZ!"
// #endif
//   RCC->CFGR = tmp;  /* clock configuration register */
//   _NOP();
//   _NOP();
//   _NOP();
//   tmp = RCC->CFGR;  /* clock configuration register */
//   set_bit(tmp, 24);
//   clear_bit(tmp, 25);  /* enable MCO output on pin PA8 */
//   clear_bit(tmp, 26);
//   clear_bit(tmp, 27);
//   RCC->CFGR = tmp;  /* clock configuration register */
//   _NOP();
//   _NOP();
//   _NOP();

#if (CPU_HZ == 80000000) || (CPU_HZ == 32000000) || (CPU_HZ == 16000000) || (CPU_HZ == 16384000)
  tmp = RCC->CFGR;  /* clock configuration register */
  set_bit(tmp, 0);  /* PLL selected as system clock */
  set_bit(tmp, 1);
  RCC->CFGR = tmp;  /* clock configuration register */
#elif (CPU_HZ == 8000000) || (CPU_HZ == 8192000)
  tmp = RCC->CFGR;  /* clock configuration register */
  clear_bit(tmp, 0);/* HSE selected as system clock */
  set_bit(tmp, 1);
  RCC->CFGR = tmp;  /* clock configuration register */
#else
#error "Invalid value for CPU_HZ!"
#endif
  _NOP();
  _NOP();
  _NOP();

  SysTick_Config(SYS_TICK_TIMER_VAL);  /* ISR: sys_tick_handler, max value: 0xffffff! */
}

























