
#include "rs232.h"

#define UART_FIFO_SZ   256


/*********************** UART 1 ****************************************************************************/

volatile unsigned char uart1_rx_fifo[UART_FIFO_SZ], uart1_tx_fifo[UART_FIFO_SZ];

volatile unsigned int uart1_rx_fifo_cnt1=0, uart1_rx_fifo_cnt2=0, uart1_tx_fifo_cnt1=0, uart1_tx_fifo_cnt2=0;


void UART1_init(int brg)
{
  unsigned int tmp=0;

  USART1->CR1 = 0;  /* disable USART1 */
  USART1->CR2 = 0;
  USART1->CR3 = 0;

  uart1_rx_fifo_cnt1 = 0;
  uart1_rx_fifo_cnt2 = 0;
  uart1_tx_fifo_cnt1 = 0;
  uart1_tx_fifo_cnt2 = 0;

  /* configure GPIO PB6 for USART1 TX configure GPIO PB7 for USART1 RX */
  /* PB3 <-> RTS  PB4 <-> CTS */
  tmp = GPIOB->MODER;    /* get mode */
//   clear_bit(tmp, 6);    /* set PB3 as alternate function */
//   set_bit(tmp, 7);
//   clear_bit(tmp, 8);    /* set PB4 as alternate function */
//   set_bit(tmp, 9);
  clear_bit(tmp, 12);    /* set PB6 as alternate function */
  set_bit(tmp, 13);
  clear_bit(tmp, 14);    /* set PB7 as alternate function */
  set_bit(tmp, 15);
  GPIOB->MODER = tmp;    /* set mode */

  tmp = GPIOB->OTYPER;   /* get output type */
  clear_bit(tmp, 6);     /* pushpull */
  GPIOB->OTYPER = tmp;   /* set output type */

  tmp = GPIOB->OSPEEDR;  /* get speed */
  clear_bit(tmp, 12);    /* low speed */
  clear_bit(tmp, 13);
  GPIOB->OSPEEDR = tmp;  /* set speed */

  tmp = GPIOB->PUPDR;    /* get pull-up/down */
  clear_bit(tmp, 14);    /* no pull-up/down */
  clear_bit(tmp, 15);
  GPIOB->PUPDR = tmp;    /* set pull-up/down */

  tmp = GPIOB->AFR[0];    /* get alternate function register high */
//   set_bit(tmp, 12);
//   set_bit(tmp, 13);      /* set PB3 to AF7 */
//   set_bit(tmp, 14);
//   clear_bit(tmp, 15);
//   set_bit(tmp, 16);
//   set_bit(tmp, 17);      /* set PB4 to AF7 */
//   set_bit(tmp, 18);
//   clear_bit(tmp, 19);
  set_bit(tmp, 24);
  set_bit(tmp, 25);      /* set PB6 to AF7 */
  set_bit(tmp, 26);
  clear_bit(tmp, 27);
  set_bit(tmp, 28);
  set_bit(tmp, 29);      /* set PB7 to AF7 */
  set_bit(tmp, 30);
  clear_bit(tmp, 31);
  GPIOB->AFR[0] = tmp;    /* set alternate function register high */

  set_bit(RCC->APB2ENR, 14);  /* enable USART1 clock */
  _NOP();
  _NOP();
  _NOP();

  set_bit(RCC->AHB1ENR, 0);  /* enable DMA1 clock */
  _NOP();
  _NOP();
  _NOP();

  tmp = RCC->CCIPR;
  clear_bit(tmp, 0);
  set_bit(tmp, 1);  /* select HSI16 clock for USART1 */
  RCC->CCIPR = tmp;
  _NOP();
  _NOP();
  _NOP();

  USART1->BRR = brg;  /* set baudrate */

//  set_bit_pbb(&USART1->CR3, 12);  /* disable overrun detection */
//   set_bit_pbb(&USART1->CR3, 8);  /* enable RTS hardware flow control */
//   set_bit_pbb(&USART1->CR3, 9);  /* enable CTS hardware flow control */

  set_bit_pbb(&USART1->CR3, 6);  /* enable DMA receiver */

  set_bit_pbb(&USART1->CR1, 0);  /* enable USART1 */
  set_bit_pbb(&USART1->CR1, 3);  /* enable TX */
  set_bit_pbb(&USART1->CR1, 2);  /* enable RX */
//  set_bit_pbb(&USART1->CR1, 5);  /* enable RX interrupts */
  set_bit_pbb(&USART1->CR1, 7);  /* enable TX interrupts */

  tmp = DMA1_CSELR->CSELR;
  clear_bit(tmp, 16);
  set_bit(tmp, 17);     /* channel 5 mapped on USART1_RX */
  clear_bit(tmp, 18);
  clear_bit(tmp, 19);
  DMA1_CSELR->CSELR = tmp;

  DMA1_Channel5->CPAR = (unsigned int)&USART1->RDR;   /* set peripheral address */
  DMA1_Channel5->CMAR = (unsigned int)uart1_rx_fifo;  /* set memory address */
  DMA1_Channel5->CNDTR = UART_FIFO_SZ;  /* set number of bytes to transfer */

  tmp = DMA1_Channel5->CCR;
  set_bit(tmp, 5);  /* enable circular mode */
  set_bit(tmp, 7);  /* enable memory increment mode */
  DMA1_Channel5->CCR = tmp;

  set_bit_pbb(&DMA1_Channel5->CCR, 0);  /* enable DMA channel */

  NVIC_EnableIRQ(USART1_IRQn);
}


void UART1_stop(void)
{
  NVIC_DisableIRQ(USART1_IRQn);
  NVIC_DisableIRQ(USART1_IRQn);

  DMA1_Channel5->CCR = 0;

  USART1->CR1 = 0;  /* disable USART1 */

  uart1_rx_fifo_cnt1 = 0;
  uart1_rx_fifo_cnt2 = 0;
  uart1_tx_fifo_cnt1 = 0;
  uart1_tx_fifo_cnt2 = 0;
}


void UART1_rx_isr_sub(void)
{
  while(test_bit(USART1->ISR, 5))  /* RX FIFO has data available? */
  {
    uart1_rx_fifo[uart1_rx_fifo_cnt1] = USART1->RDR;  /* copy byte from uart FIFO */

    if(++uart1_rx_fifo_cnt1 >= UART_FIFO_SZ)  uart1_rx_fifo_cnt1 = 0;
  }
}


void UART1_tx_isr_sub(void)          /* uart TX interrupt */
{
  while(test_bit(USART1->ISR, 7))  /* uart FIFO not full? */
  {
    if(uart1_tx_fifo_cnt1 != uart1_tx_fifo_cnt2)
    {
      USART1->TDR = uart1_tx_fifo[uart1_tx_fifo_cnt2];  /* copy byte to uart FIFO */

      if(++uart1_tx_fifo_cnt2 >= UART_FIFO_SZ)  uart1_tx_fifo_cnt2 = 0;
    }
    else
    {
      clear_bit_pbb(&USART1->CR1, 7);  /* disable TX interrupts */
      clear_bit_pbb(&USART1->CR1, 7);  /* disable TX interrupts */

      break;
    }
  }
}


void USART1_isr_handler(void)  /* function name must be the same as used in interrupt vector table in file startup_stm32.s! */
{
  if(test_bit(USART1->ISR, 5))  /* Which interrupt is pending, RX? */
  {
    UART1_rx_isr_sub();
  }
  else if(test_bit(USART1->ISR, 7))  /* Which interrupt is pending, TX? */
    {
      UART1_tx_isr_sub();
    }
    else if(test_bit(USART1->ISR, 3))  /* an overrun error occurred? */
      {
        clear_bit_pbb(&USART1->ICR, 3);  /* clear overrun error flag */
      }
}


void UART1_putch(char byte)    /* sends a byte to serial port */
{
BEGIN_PUTCH:

  if(!test_bit(USART1->CR1, 0))  return;

  if(uart1_tx_fifo_cnt1 > uart1_tx_fifo_cnt2)
  {
    if((uart1_tx_fifo_cnt1 - uart1_tx_fifo_cnt2) > UART_FIFO_SZ - 2)  goto BEGIN_PUTCH;  /* buffer full */
  }

  if(uart1_tx_fifo_cnt1 < uart1_tx_fifo_cnt2)
  {
    if((uart1_tx_fifo_cnt2 - uart1_tx_fifo_cnt1) < 2)  goto BEGIN_PUTCH;  /* buffer full */
  }

  clear_bit_pbb(&USART1->CR1, 7);  /* disable TX interrupts */
  clear_bit_pbb(&USART1->CR1, 7);  /* disable TX interrupts */

  uart1_tx_fifo[uart1_tx_fifo_cnt1] = byte;

  if(++uart1_tx_fifo_cnt1 >= UART_FIFO_SZ)  uart1_tx_fifo_cnt1 = 0;

  set_bit_pbb(&USART1->CR1, 7);  /* enable TX interrupts */
}


void UART1_flush_txbuf(void)  /* flush the TX buffer */
{
  clear_bit_pbb(&USART1->CR1, 7);  /* disable TX interrupts */
  clear_bit_pbb(&USART1->CR1, 7);  /* disable TX interrupts */

  uart1_tx_fifo_cnt1 = uart1_tx_fifo_cnt2;

  set_bit_pbb(&USART1->CR1, 7);  /* enable TX interrupts */
}


unsigned int UART1_txbuf_stat(void)  /*  checks serial port transmit buffer,     */
{                                    /*  how much space left in transmit buffer? */
  if(!test_bit(USART1->CR1, 0))  return 0;

  if(uart1_tx_fifo_cnt1 == uart1_tx_fifo_cnt2)
  {
    return UART_FIFO_SZ - 2;
  }
  else if(uart1_tx_fifo_cnt1 >= uart1_tx_fifo_cnt2)
    {
      return UART_FIFO_SZ - (uart1_tx_fifo_cnt1 - uart1_tx_fifo_cnt2) - 2;
    }
    else
    {
      return UART_FIFO_SZ - (uart1_tx_fifo_cnt2 - uart1_tx_fifo_cnt1) - 2;
    }
}


unsigned int UART1_kbhit(void)                /*  checks serial port receiver, byte arrived? */
{
  uart1_rx_fifo_cnt1 = UART_FIFO_SZ - DMA1_Channel5->CNDTR;

  if(uart1_rx_fifo_cnt1 != uart1_rx_fifo_cnt2)   return 1;
  else return 0;
}


char UART1_getch(void)                /* receives a byte from the serial port */
{
  char byte;

  byte = uart1_rx_fifo[uart1_rx_fifo_cnt2];
  if(++uart1_rx_fifo_cnt2 >= UART_FIFO_SZ)  uart1_rx_fifo_cnt2 = 0;
  return byte;
}


int UART1_read(void *dest, int len)
{
  int i, sz=0;

  if(len < 1)  return 0;

  uart1_rx_fifo_cnt1 = UART_FIFO_SZ - DMA1_Channel5->CNDTR;

  if(uart1_rx_fifo_cnt1 == uart1_rx_fifo_cnt2)   return 0;

  sz = uart1_rx_fifo_cnt1 - uart1_rx_fifo_cnt2;

  if(sz < 0)  sz += UART_FIFO_SZ;

  if(sz > len)  sz = len;

  for(i=0; i<sz; i++)
  {
    ((char *)dest)[i] = uart1_rx_fifo[uart1_rx_fifo_cnt2];

    if(++uart1_rx_fifo_cnt2 >= UART_FIFO_SZ)  uart1_rx_fifo_cnt2 = 0;
  }

  return sz;
}


void UART1_cprintf(const void *str)  /* sends a string to serial port */
{
  unsigned char *ptr_str=(unsigned char *)str;

  while(*ptr_str != 0)  UART1_putch(*(ptr_str++));
}


void UART1_print_dec(unsigned char val)
{
  unsigned char flag=0, z=0, i=0, base = 100;

  if(!val)
  {
    UART1_putch('0');
    return;
  }

  for(i=3; i; i--)
  {
    z = val / base;
    val %= base;
    if(z || flag)
    {
      UART1_putch('0' + z);
      flag = 1;
    }
/*    else  UART1_putch(' '); */
    base /= 10;
  }
}


void UART1_print_short_dec(unsigned short q)
{
  unsigned char flag=0, z=0, i=0;

  unsigned short base = 10000;

  if(!q)
  {
    UART1_putch('0');
    return;
  }

  for(i=5; i; i--)
  {
    z = q / base;
    q %= base;
    if(z||flag)
    {
      UART1_putch('0' + z);
      flag = 1;
    }
//    else  UART1_putch(' ');
    base /= 10;
  }
}


void UART1_print_long_dec(unsigned int q)
{
  unsigned char flag=0, z=0, i=0;

  unsigned int base = 1000000000;

  if(!q)
  {
    UART1_putch('0');
    return;
  }

  for(i=10; i; i--)
  {
    z = q / base;
    q %= base;
    if(z||flag)
    {
      UART1_putch('0' + z);
      flag = 1;
    }
//    else  UART1_putch(' ');
    base /= 10;
  }
}


void UART1_print_signed_long_dec(int q)
{
  unsigned char flag=0, z=0, i=0;

  unsigned int base = 1000000000;

  if(!q)
  {
    UART1_putch('0');
    return;
  }

  if(q<0)
  {
    UART1_putch('-');

    q *= -1;
  }

  for(i=10; i; i--)
  {
    z = q / base;
    q %= base;
    if(z||flag)
    {
      UART1_putch('0' + z);
      flag = 1;
    }
//    else  UART1_putch(' ');
    base /= 10;
  }
}


int UART1_print_hex_lz(unsigned int val, int n_digits)
{
  int flag=0, z, i, j=0;

  unsigned int base = 0x10000000;

  if(n_digits < 1)  n_digits = 1;

  if(n_digits > 8)  n_digits = 8;

  if(!val)
  {
    for(i=0; i<n_digits; i++)
    {
      UART1_putch('0');
    }

    return n_digits;
  }

  for(i=0; i<8; i++)
  {
    if(i == (8 - n_digits))
    {
      flag = 1;
    }

    z = val / base;

    val = val % base;

    if(z || flag)
    {
      if(z < 10)
      {
        UART1_putch('0' + z);
      }
      else
      {
        UART1_putch('7' + z);
      }

      flag = 1;
    }

    base /= 0x10;
  }

  return j;
}













